/*!
  @file UcglibI2C.h
  @brief An I2Cwrapper module implementing (a subset of) the great Ucglib by Oli 
  Kraus, so that you can control any TFT display supported by Ucglib via I2C.
    
  ## Author
  Copyright (c) 2023 juh
  ## License
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License as
  published by the Free Software Foundation, version 2.
*/


#ifndef UcglibI2C_h
#define UcglibI2C_h

// #define DEBUG // uncomment for serial debugging, don't forget Serial.begin() in your controller's setup()

#include <Arduino.h>
#include <Print.h>
#include <Ucglib.h>
#include <I2Cwrapper.h>


#if !defined(log)
#if defined(DEBUG)
#define log(...)       Serial.print(__VA_ARGS__)
#else
#define log(...)
#endif // DEBUG
#endif // log


// Ucglib commands (reserved  090 - 109 UcglibI2C)
/* Note we're trying the idea of subcommands, here. The idea is to group together 
 * commands which share the same list of parameters. It'll save on uniqe byte 
 * command codes and should slightly decrease code size, saving memory for fonts.
 * However, it will slightly increase I2C traffic and execution times. That's why 
 * timing critical and often called commands like write() retain their own codes 
 */
const uint8_t UcglibCmdOffset = 90;
const uint8_t UcglibBeginCmd        = UcglibCmdOffset + 0;
const uint8_t UcglibClearScreenCmd  = UcglibCmdOffset + 1;
const uint8_t UcglibSetFontCmd      = UcglibCmdOffset + 2;
const uint8_t UcglibSetColorCmd     = UcglibCmdOffset + 3;
const uint8_t UcglibSetPrintPosCmd  = UcglibCmdOffset + 4;
// print() is implemented locally by overwriting write() method, so no command needed
const uint8_t UcglibWriteCmd        = UcglibCmdOffset + 5;

const uint8_t UcglibSettingCmd      = UcglibCmdOffset + 6;
const uint8_t UcglibSettingCmdRotate0                      = 0; // "subcommands" to save on unique command numbers
const uint8_t UcglibSettingCmdRotate90                     = 1;
const uint8_t UcglibSettingCmdRotate180                    = 2;
const uint8_t UcglibSettingCmdRotate270                    = 3;
const uint8_t UcglibSettingCmdSetFontRefHeightText         = 4;
const uint8_t UcglibSettingCmdSetFontRefHeightExtendedText = 5;
const uint8_t UcglibSettingCmdSetFontRefHeightAll          = 6;
const uint8_t UcglibSettingCmdSetFontPosBaseline           = 7;
const uint8_t UcglibSettingCmdSetFontPosBottom             = 8;
const uint8_t UcglibSettingCmdSetFontPosTop                = 9;
const uint8_t UcglibSettingCmdSetFontPosCenter             = 10;
const uint8_t UcglibSettingCmdUndoScale                    = 11;
const uint8_t UcglibSettingCmdSetScale2x2                  = 12;
const uint8_t UcglibSettingCmdPowerDown                    = 13;
const uint8_t UcglibSettingCmdPowerUp                      = 14;
const uint8_t UcglibSettingCmdSetMaxClipRange              = 15;
const uint8_t UcglibSettingCmdUndoClipRange                = 16;

const uint8_t Ucglib1uint8_tCmd     = UcglibCmdOffset + 7;
const uint8_t Ucglib1uint8_tCmdSetPrintDir                 = 0;
const uint8_t Ucglib1uint8_tCmdSetFontMode                 = 1;

const uint8_t UcglibGetCmd          = UcglibCmdOffset + 8; const uint8_t UcglibGetCmdResult = sizeof(ucg_int_t);
const uint8_t UcglibGetCmdGetWidth                         = 0;
const uint8_t UcglibGetCmdGetHeight                        = 1;
const uint8_t UcglibGetCmdGetFontAscent                    = 2;
const uint8_t UcglibGetCmdGetFontDescent                   = 3;

const uint8_t UcglibGetStrWidthCmd  = UcglibCmdOffset + 9; const uint8_t UcglibGetStrWidthCmdResult = sizeof(ucg_int_t);

const uint8_t Ucglib4ucg_int_tCmd   = UcglibCmdOffset + 10;
const uint8_t Ucglib4ucg_int_tCmdSetClipRange              = 0;
const uint8_t Ucglib4ucg_int_tCmdDrawLine                  = 1;
const uint8_t Ucglib4ucg_int_tCmdDrawBox                   = 2;
const uint8_t Ucglib4ucg_int_tCmdDrawFrame                 = 3;
const uint8_t Ucglib4ucg_int_tCmdDrawGradientLine          = 4;
const uint8_t Ucglib4ucg_int_tCmdDrawGradientBox           = 5;

const uint8_t UcglibDrawPixelCmd    = UcglibCmdOffset + 11;

const uint8_t Ucglib3ucg_int_tCmd   = UcglibCmdOffset + 12;
const uint8_t Ucglib3ucg_int_tCmdDrawHLine                 = 0;
const uint8_t Ucglib3ucg_int_tCmdDrawVLine                 = 1;

const uint8_t Ucglib5ucg_int_tCmd   = UcglibCmdOffset + 13;
const uint8_t Ucglib5ucg_int_tCmdDrawRBox                  = 0;
const uint8_t Ucglib5ucg_int_tCmdDrawRFrame                = 1;

const uint8_t UcglibDrawGlyphCmd    = UcglibCmdOffset + 14; const uint8_t UcglibDrawGlyphCmdResult = sizeof(ucg_int_t);

const uint8_t UcglibDrawStringCmd   = UcglibCmdOffset + 15; const uint8_t UcglibDrawStringCmdResult = sizeof(ucg_int_t);

const uint8_t UcglibDrawWithRadiusCmd   = UcglibCmdOffset + 16;
const uint8_t UcglibDrawWithRadiusCmdDrawDisc              = 0;
const uint8_t UcglibDrawWithRadiusCmdDrawCircle            = 1;

const uint8_t UcglibDrawTriangleCmd = UcglibCmdOffset + 17;

const uint8_t UcglibDrawTetragonCmd = UcglibCmdOffset + 18;

//const uint8_t Ucglib...Cmd     = UcglibCmdOffset + 19;


enum UcglibI2C_Font : uint16_t; // forward declaration so we can move the clutter to the end of the file


/*!
  @brief An I2C wrapper class for Ucglib.
  @details UcglibI2C implements, with only minor restrictions, the full 
  functionality of [Ucglib by Oli Kraus](https://github.com/olikraus/ucglib),
  so that you can control any TFT or other display supported by Ucglib via I2C.
  
  ### Limitations (also see README.md):
  
  - Timing: Some Ucglib commands take so long that even the conservative default for 
  I2Cdelay of 20 ms is not sufficient. They need extra delays or you'll run 
  the risk of commands being dropped or even I2C bus lockup. Particularly watch 
  out for begin() and clearScreen(), also for all commands changing a lot of 
  pixels like large boxes, triangles etc. Start with extra delays of 200 ms 
  and work down from that.
  
  - Strings: drawString() and getStrWidth() commands are limited by I2CmaxBuf, 
  which currently can only be directly modified in I2CWrapper.h, until a 
  more flexible mechanism will be implemented. Due to overhead, max string 
  length is (I2CmaxBuf - 10) = 10 characters for drawString() and 
  (I2CmaxBuf - 6) = 14 characters for getStrWidth(). print() is not restricted,
  as it is inherited from the Arduino 
  [Print class](https://github.com/arduino/ArduinoCore-avr/blob/master/cores/arduino/Print.h).
  
  - Fonts: Ucglib uses pointers to identify fonts. Here, we cannot transmit pointers 
  to fonts for three reasons:
   (1) Addresses on the target will be different.
   (2) We do not want the controller to waste memory for fonts only the target will need.
   (3) The target might not have enough memory to make all Ucglib fonts available at all times.   
   That's why the target will have a preselected subset of fonts available and we
   only transmit the ID (of enum type UcglibI2C_Font) of that font here. ID's are nearly 
   identical to the original font names, which are preceded with "I2C_", so that only 
   minimal changes to any original code will be needed.
   
  */

class UcglibI2C : public Print // derived from print, else sending strings around would be awkward
{
public:
  /*!
   * @brief Constructor.
   * @param w Wrapper object representing the target the TFT/Ucglib display is connected to.
   */
  UcglibI2C(I2Cwrapper* w);
  
  void begin(uint8_t is_transparent);
  void clearScreen(void);
  
  void setFont(UcglibI2C_Font id);
  void setColor(uint8_t idx, uint8_t r, uint8_t g, uint8_t b);
  void setColor(uint8_t r, uint8_t g, uint8_t b);
  void setPrintPos(ucg_int_t x, ucg_int_t y);
  size_t write(uint8_t c); 
  /// @private
  void settingCmd(uint8_t subcommand);
  void undoRotate();
  void setRotate90();
  void setRotate180();
  void setRotate270();
  void setFontRefHeightText();
  void setFontRefHeightExtendedText();
  void setFontRefHeightAll();
  void setFontPosBaseline();
  void setFontPosBottom();
  void setFontPosTop();
  void setFontPosCenter();
  void undoScale();
  void setScale2x2();
  void powerDown();
  void powerUp();
  void setMaxClipRange();
  void undoClipRange();
  /// @private
  void OneUint8_tCmd(uint8_t subcommand, uint8_t p1);
  void setPrintDir(uint8_t dir);
  void setFontMode(uint8_t is_transparent);
  /// @private
  ucg_int_t GetCmd(uint8_t subcommand);
  ucg_int_t getWidth();
  ucg_int_t getHeight();
  ucg_int_t getFontAscent();
  ucg_int_t getFontDescent();
  ucg_int_t getStrWidth(const char *s);
  /// @private
  void FourUcg_int_tCmd(uint8_t subcommand, ucg_int_t p1, ucg_int_t p2, ucg_int_t p3, ucg_int_t p4);
  void setClipRange (ucg_int_t x,  ucg_int_t y,  ucg_int_t w,   ucg_int_t h);
  void drawLine (ucg_int_t x1, ucg_int_t y1, ucg_int_t x2,  ucg_int_t y2);
  void drawBox (ucg_int_t x,  ucg_int_t y,  ucg_int_t w,   ucg_int_t h);
  void drawFrame (ucg_int_t x,  ucg_int_t y,  ucg_int_t w,   ucg_int_t h);
  void drawGradientLine (ucg_int_t x,  ucg_int_t y,  ucg_int_t len, ucg_int_t dir);
  void drawGradientBox (ucg_int_t x,  ucg_int_t y,  ucg_int_t w,   ucg_int_t h);
  void drawPixel(ucg_int_t x, ucg_int_t y);
  /// @private
  void ThreeUcg_int_tCmd(uint8_t subcommand, ucg_int_t p1, ucg_int_t p2, ucg_int_t p3);
  void drawHLine(ucg_int_t x,  ucg_int_t y,  ucg_int_t len);
  void drawVLine(ucg_int_t x,  ucg_int_t y,  ucg_int_t len);
  /// @private
  void FiveUcg_int_tCmd(uint8_t subcommand, ucg_int_t p1, ucg_int_t p2, ucg_int_t p3, ucg_int_t p4, ucg_int_t p5);
  void drawRBox(ucg_int_t x, ucg_int_t y, ucg_int_t w, ucg_int_t h, ucg_int_t r);
  void drawRFrame(ucg_int_t x, ucg_int_t y, ucg_int_t w, ucg_int_t h, ucg_int_t r);
  ucg_int_t drawGlyph(ucg_int_t x, ucg_int_t y, uint8_t dir, uint8_t encoding);
  ucg_int_t drawString(ucg_int_t x, ucg_int_t y, uint8_t dir, const char *str);
  /// @private
  void DrawWithRadius(uint8_t subcommand, ucg_int_t x0, ucg_int_t y0, ucg_int_t rad, uint8_t option);
  void drawDisc (ucg_int_t x0, ucg_int_t y0, ucg_int_t rad, uint8_t option);
  void drawCircle(ucg_int_t x0, ucg_int_t y0, ucg_int_t rad, uint8_t option);
  void drawTriangle(int16_t x0, int16_t y0, int16_t x1, int16_t y1, int16_t x2, int16_t y2);
  void drawTetragon(int16_t x0, int16_t y0, int16_t x1, int16_t y1, int16_t x2, int16_t y2, int16_t x3, int16_t y3);

  
  /* ************************************************************************************ */
/*    

  // not implemented
  //ucg_t *getUcg(void);
  
*/
/* ************************************************************************************ */
  
  //int8_t myNum = -1;    // for modules that support units, i.e. more than one instance of the hardware represented by this class
  
private:
  I2Cwrapper* wrapper;
};



// enum used for ID'ing fonts over I2C
enum UcglibI2C_Font : uint16_t { // we need to define the data type to allow for safety checks when receiving data of type UcglibI2C_Font by the target
  I2C_ucg_font_04b_03,
  I2C_ucg_font_04b_03b,
  I2C_ucg_font_04b_03b_hf,
  I2C_ucg_font_04b_03b_hn,
  I2C_ucg_font_04b_03b_hr,
  I2C_ucg_font_04b_03bn,
  I2C_ucg_font_04b_03br,
  I2C_ucg_font_04b_03b_tf,
  I2C_ucg_font_04b_03b_tn,
  I2C_ucg_font_04b_03b_tr,
  I2C_ucg_font_04b_03_hf,
  I2C_ucg_font_04b_03_hn,
  I2C_ucg_font_04b_03_hr,
  I2C_ucg_font_04b_03n,
  I2C_ucg_font_04b_03r,
  I2C_ucg_font_04b_03_tf,
  I2C_ucg_font_04b_03_tn,
  I2C_ucg_font_04b_03_tr,
  I2C_ucg_font_04b_24,
  I2C_ucg_font_04b_24_hf,
  I2C_ucg_font_04b_24_hn,
  I2C_ucg_font_04b_24_hr,
  I2C_ucg_font_04b_24n,
  I2C_ucg_font_04b_24r,
  I2C_ucg_font_04b_24_tf,
  I2C_ucg_font_04b_24_tn,
  I2C_ucg_font_04b_24_tr,
  I2C_ucg_font_10x20,
  I2C_ucg_font_10x20_75r,
  I2C_ucg_font_10x20_mf,
  I2C_ucg_font_10x20_mr,
  I2C_ucg_font_10x20r,
  I2C_ucg_font_10x20_tf,
  I2C_ucg_font_10x20_tr,
  I2C_ucg_font_4x6,
  I2C_ucg_font_4x6_mf,
  I2C_ucg_font_4x6_mr,
  I2C_ucg_font_4x6r,
  I2C_ucg_font_4x6_tf,
  I2C_ucg_font_4x6_tr,
  I2C_ucg_font_5x7,
  I2C_ucg_font_5x7_8f,
  I2C_ucg_font_5x7_8r,
  I2C_ucg_font_5x7_mf,
  I2C_ucg_font_5x7_mr,
  I2C_ucg_font_5x7r,
  I2C_ucg_font_5x7_tf,
  I2C_ucg_font_5x7_tr,
  I2C_ucg_font_5x8,
  I2C_ucg_font_5x8_8f,
  I2C_ucg_font_5x8_8r,
  I2C_ucg_font_5x8_mf,
  I2C_ucg_font_5x8_mr,
  I2C_ucg_font_5x8r,
  I2C_ucg_font_5x8_tf,
  I2C_ucg_font_5x8_tr,
  I2C_ucg_font_6x10,
  I2C_ucg_font_6x10_mf,
  I2C_ucg_font_6x10_mr,
  I2C_ucg_font_6x10r,
  I2C_ucg_font_6x10_tf,
  I2C_ucg_font_6x10_tr,
  I2C_ucg_font_6x12,
  I2C_ucg_font_6x12_75,
  I2C_ucg_font_6x12_75r,
  I2C_ucg_font_6x12_mf,
  I2C_ucg_font_6x12_mr,
  I2C_ucg_font_6x12r,
  I2C_ucg_font_6x12_tf,
  I2C_ucg_font_6x12_tr,
  I2C_ucg_font_6x13,
  I2C_ucg_font_6x13_75r,
  I2C_ucg_font_6x13B,
  I2C_ucg_font_6x13Br,
  I2C_ucg_font_6x13B_tf,
  I2C_ucg_font_6x13B_tr,
  I2C_ucg_font_6x13_mf,
  I2C_ucg_font_6x13_mr,
  I2C_ucg_font_6x13O,
  I2C_ucg_font_6x13Or,
  I2C_ucg_font_6x13O_tf,
  I2C_ucg_font_6x13O_tr,
  I2C_ucg_font_6x13r,
  I2C_ucg_font_6x13_tf,
  I2C_ucg_font_6x13_tr,
  I2C_ucg_font_7x13,
  I2C_ucg_font_7x13_75r,
  I2C_ucg_font_7x13_78_79,
  I2C_ucg_font_7x13B,
  I2C_ucg_font_7x13Br,
  I2C_ucg_font_7x13B_tf,
  I2C_ucg_font_7x13B_tr,
  I2C_ucg_font_7x13_mf,
  I2C_ucg_font_7x13_mr,
  I2C_ucg_font_7x13O,
  I2C_ucg_font_7x13Or,
  I2C_ucg_font_7x13O_tf,
  I2C_ucg_font_7x13O_tr,
  I2C_ucg_font_7x13r,
  I2C_ucg_font_7x13_tf,
  I2C_ucg_font_7x13_tr,
  I2C_ucg_font_7x14,
  I2C_ucg_font_7x14B,
  I2C_ucg_font_7x14B_mf,
  I2C_ucg_font_7x14B_mr,
  I2C_ucg_font_7x14Br,
  I2C_ucg_font_7x14B_tf,
  I2C_ucg_font_7x14B_tr,
  I2C_ucg_font_7x14_mf,
  I2C_ucg_font_7x14_mr,
  I2C_ucg_font_7x14r,
  I2C_ucg_font_7x14_tf,
  I2C_ucg_font_7x14_tr,
  I2C_ucg_font_8x13,
  I2C_ucg_font_8x13_75r,
  I2C_ucg_font_8x13B,
  I2C_ucg_font_8x13B_mf,
  I2C_ucg_font_8x13B_mr,
  I2C_ucg_font_8x13Br,
  I2C_ucg_font_8x13B_tf,
  I2C_ucg_font_8x13B_tr,
  I2C_ucg_font_8x13_mf,
  I2C_ucg_font_8x13_mr,
  I2C_ucg_font_8x13O,
  I2C_ucg_font_8x13O_mf,
  I2C_ucg_font_8x13O_mr,
  I2C_ucg_font_8x13Or,
  I2C_ucg_font_8x13O_tf,
  I2C_ucg_font_8x13O_tr,
  I2C_ucg_font_8x13r,
  I2C_ucg_font_8x13_tf,
  I2C_ucg_font_8x13_tr,
  I2C_ucg_font_9x15,
  I2C_ucg_font_9x15_75r,
  I2C_ucg_font_9x15B,
  I2C_ucg_font_9x15B_mf,
  I2C_ucg_font_9x15B_mr,
  I2C_ucg_font_9x15Br,
  I2C_ucg_font_9x15B_tf,
  I2C_ucg_font_9x15B_tr,
  I2C_ucg_font_9x15_mf,
  I2C_ucg_font_9x15_mr,
  I2C_ucg_font_9x15r,
  I2C_ucg_font_9x15_tf,
  I2C_ucg_font_9x15_tr,
  I2C_ucg_font_9x18,
  I2C_ucg_font_9x18_75r,
  I2C_ucg_font_9x18B,
  I2C_ucg_font_9x18B_mf,
  I2C_ucg_font_9x18B_mr,
  I2C_ucg_font_9x18Br,
  I2C_ucg_font_9x18B_tf,
  I2C_ucg_font_9x18B_tr,
  I2C_ucg_font_9x18_mf,
  I2C_ucg_font_9x18_mr,
  I2C_ucg_font_9x18r,
  I2C_ucg_font_9x18_tf,
  I2C_ucg_font_9x18_tr,
  I2C_ucg_font_amstrad_cpc_8f,
  I2C_ucg_font_amstrad_cpc_8r,
  I2C_ucg_font_baby,
  I2C_ucg_font_baby_hf,
  I2C_ucg_font_baby_hn,
  I2C_ucg_font_baby_hr,
  I2C_ucg_font_babyn,
  I2C_ucg_font_babyr,
  I2C_ucg_font_baby_tf,
  I2C_ucg_font_baby_tn,
  I2C_ucg_font_baby_tr,
  I2C_ucg_font_blipfest_07,
  I2C_ucg_font_blipfest_07_hf,
  I2C_ucg_font_blipfest_07_hn,
  I2C_ucg_font_blipfest_07_hr,
  I2C_ucg_font_blipfest_07n,
  I2C_ucg_font_blipfest_07r,
  I2C_ucg_font_blipfest_07_tf,
  I2C_ucg_font_blipfest_07_tn,
  I2C_ucg_font_blipfest_07_tr,
  I2C_ucg_font_chikita,
  I2C_ucg_font_chikita_hf,
  I2C_ucg_font_chikita_hn,
  I2C_ucg_font_chikita_hr,
  I2C_ucg_font_chikitan,
  I2C_ucg_font_chikitar,
  I2C_ucg_font_chikita_tf,
  I2C_ucg_font_chikita_tn,
  I2C_ucg_font_chikita_tr,
  I2C_ucg_font_courB08,
  I2C_ucg_font_courB08_mf,
  I2C_ucg_font_courB08_mr,
  I2C_ucg_font_courB08r,
  I2C_ucg_font_courB08_tf,
  I2C_ucg_font_courB08_tr,
  I2C_ucg_font_courB10,
  I2C_ucg_font_courB10_mf,
  I2C_ucg_font_courB10_mr,
  I2C_ucg_font_courB10r,
  I2C_ucg_font_courB10_tf,
  I2C_ucg_font_courB10_tr,
  I2C_ucg_font_courB12,
  I2C_ucg_font_courB12_mf,
  I2C_ucg_font_courB12_mr,
  I2C_ucg_font_courB12r,
  I2C_ucg_font_courB12_tf,
  I2C_ucg_font_courB12_tr,
  I2C_ucg_font_courB14,
  I2C_ucg_font_courB14_mf,
  I2C_ucg_font_courB14_mr,
  I2C_ucg_font_courB14r,
  I2C_ucg_font_courB14_tf,
  I2C_ucg_font_courB14_tr,
  I2C_ucg_font_courB18,
  I2C_ucg_font_courB18_mf,
  I2C_ucg_font_courB18_mn,
  I2C_ucg_font_courB18_mr,
  I2C_ucg_font_courB18r,
  I2C_ucg_font_courB18_tf,
  I2C_ucg_font_courB18_tn,
  I2C_ucg_font_courB18_tr,
  I2C_ucg_font_courB24,
  I2C_ucg_font_courB24_mf,
  I2C_ucg_font_courB24_mn,
  I2C_ucg_font_courB24_mr,
  I2C_ucg_font_courB24n,
  I2C_ucg_font_courB24r,
  I2C_ucg_font_courB24_tf,
  I2C_ucg_font_courB24_tn,
  I2C_ucg_font_courB24_tr,
  I2C_ucg_font_courR08,
  I2C_ucg_font_courR08_mf,
  I2C_ucg_font_courR08_mr,
  I2C_ucg_font_courR08r,
  I2C_ucg_font_courR08_tf,
  I2C_ucg_font_courR08_tr,
  I2C_ucg_font_courR10,
  I2C_ucg_font_courR10_mf,
  I2C_ucg_font_courR10_mr,
  I2C_ucg_font_courR10r,
  I2C_ucg_font_courR10_tf,
  I2C_ucg_font_courR10_tr,
  I2C_ucg_font_courR12,
  I2C_ucg_font_courR12_mf,
  I2C_ucg_font_courR12_mr,
  I2C_ucg_font_courR12r,
  I2C_ucg_font_courR12_tf,
  I2C_ucg_font_courR12_tr,
  I2C_ucg_font_courR14,
  I2C_ucg_font_courR14_mf,
  I2C_ucg_font_courR14_mr,
  I2C_ucg_font_courR14r,
  I2C_ucg_font_courR14_tf,
  I2C_ucg_font_courR14_tr,
  I2C_ucg_font_courR18,
  I2C_ucg_font_courR18_mf,
  I2C_ucg_font_courR18_mr,
  I2C_ucg_font_courR18r,
  I2C_ucg_font_courR18_tf,
  I2C_ucg_font_courR18_tr,
  I2C_ucg_font_courR24,
  I2C_ucg_font_courR24_mf,
  I2C_ucg_font_courR24_mn,
  I2C_ucg_font_courR24_mr,
  I2C_ucg_font_courR24n,
  I2C_ucg_font_courR24r,
  I2C_ucg_font_courR24_tf,
  I2C_ucg_font_courR24_tn,
  I2C_ucg_font_courR24_tr,
  I2C_ucg_font_cu12,
  I2C_ucg_font_cu12_75,
  I2C_ucg_font_cu12_75r,
  I2C_ucg_font_cu12_hf,
  I2C_ucg_font_cu12_mf,
  I2C_ucg_font_cu12_tf,
  I2C_ucg_font_cursor,
  I2C_ucg_font_cursorr,
  I2C_ucg_font_cursor_tf,
  I2C_ucg_font_cursor_tr,
  I2C_ucg_font_fixed_v0,
  I2C_ucg_font_fixed_v0_hf,
  I2C_ucg_font_fixed_v0_hn,
  I2C_ucg_font_fixed_v0_hr,
  I2C_ucg_font_fixed_v0_mr,
  I2C_ucg_font_fixed_v0n,
  I2C_ucg_font_fixed_v0r,
  I2C_ucg_font_fixed_v0_tf,
  I2C_ucg_font_fixed_v0_tn,
  I2C_ucg_font_fixed_v0_tr,
  I2C_ucg_font_freedoomr10r,
  I2C_ucg_font_freedoomr10_tr,
  I2C_ucg_font_freedoomr25n,
  I2C_ucg_font_freedoomr25_tn,
  I2C_ucg_font_fub11,
  I2C_ucg_font_fub11_hf,
  I2C_ucg_font_fub11_hn,
  I2C_ucg_font_fub11_hr,
  I2C_ucg_font_fub11n,
  I2C_ucg_font_fub11r,
  I2C_ucg_font_fub11_tf,
  I2C_ucg_font_fub11_tn,
  I2C_ucg_font_fub11_tr,
  I2C_ucg_font_fub14,
  I2C_ucg_font_fub14_hf,
  I2C_ucg_font_fub14_hn,
  I2C_ucg_font_fub14_hr,
  I2C_ucg_font_fub14n,
  I2C_ucg_font_fub14r,
  I2C_ucg_font_fub14_tf,
  I2C_ucg_font_fub14_tn,
  I2C_ucg_font_fub14_tr,
  I2C_ucg_font_fub17,
  I2C_ucg_font_fub17_hf,
  I2C_ucg_font_fub17_hn,
  I2C_ucg_font_fub17_hr,
  I2C_ucg_font_fub17n,
  I2C_ucg_font_fub17r,
  I2C_ucg_font_fub17_tf,
  I2C_ucg_font_fub17_tn,
  I2C_ucg_font_fub17_tr,
  I2C_ucg_font_fub20,
  I2C_ucg_font_fub20_hf,
  I2C_ucg_font_fub20_hn,
  I2C_ucg_font_fub20_hr,
  I2C_ucg_font_fub20n,
  I2C_ucg_font_fub20r,
  I2C_ucg_font_fub20_tf,
  I2C_ucg_font_fub20_tn,
  I2C_ucg_font_fub20_tr,
  I2C_ucg_font_fub25,
  I2C_ucg_font_fub25_hf,
  I2C_ucg_font_fub25_hn,
  I2C_ucg_font_fub25_hr,
  I2C_ucg_font_fub25n,
  I2C_ucg_font_fub25r,
  I2C_ucg_font_fub25_tf,
  I2C_ucg_font_fub25_tn,
  I2C_ucg_font_fub25_tr,
  I2C_ucg_font_fub30,
  I2C_ucg_font_fub30_hf,
  I2C_ucg_font_fub30_hn,
  I2C_ucg_font_fub30_hr,
  I2C_ucg_font_fub30n,
  I2C_ucg_font_fub30r,
  I2C_ucg_font_fub30_tf,
  I2C_ucg_font_fub30_tn,
  I2C_ucg_font_fub30_tr,
  I2C_ucg_font_fub35_hf,
  I2C_ucg_font_fub35_hn,
  I2C_ucg_font_fub35_hr,
  I2C_ucg_font_fub35n,
  I2C_ucg_font_fub35_tf,
  I2C_ucg_font_fub35_tn,
  I2C_ucg_font_fub35_tr,
  I2C_ucg_font_fub42_hf,
  I2C_ucg_font_fub42_hn,
  I2C_ucg_font_fub42_hr,
  I2C_ucg_font_fub42n,
  I2C_ucg_font_fub42_tf,
  I2C_ucg_font_fub42_tn,
  I2C_ucg_font_fub42_tr,
  I2C_ucg_font_fub49_hn,
  I2C_ucg_font_fub49n,
  I2C_ucg_font_fub49_tn,
  I2C_ucg_font_fur11,
  I2C_ucg_font_fur11_hf,
  I2C_ucg_font_fur11_hn,
  I2C_ucg_font_fur11_hr,
  I2C_ucg_font_fur11n,
  I2C_ucg_font_fur11r,
  I2C_ucg_font_fur11_tf,
  I2C_ucg_font_fur11_tn,
  I2C_ucg_font_fur11_tr,
  I2C_ucg_font_fur14,
  I2C_ucg_font_fur14_hf,
  I2C_ucg_font_fur14_hn,
  I2C_ucg_font_fur14_hr,
  I2C_ucg_font_fur14n,
  I2C_ucg_font_fur14r,
  I2C_ucg_font_fur14_tf,
  I2C_ucg_font_fur14_tn,
  I2C_ucg_font_fur14_tr,
  I2C_ucg_font_fur17,
  I2C_ucg_font_fur17_hf,
  I2C_ucg_font_fur17_hn,
  I2C_ucg_font_fur17_hr,
  I2C_ucg_font_fur17n,
  I2C_ucg_font_fur17r,
  I2C_ucg_font_fur17_tf,
  I2C_ucg_font_fur17_tn,
  I2C_ucg_font_fur17_tr,
  I2C_ucg_font_fur20,
  I2C_ucg_font_fur20_hf,
  I2C_ucg_font_fur20_hn,
  I2C_ucg_font_fur20_hr,
  I2C_ucg_font_fur20n,
  I2C_ucg_font_fur20r,
  I2C_ucg_font_fur20_tf,
  I2C_ucg_font_fur20_tn,
  I2C_ucg_font_fur20_tr,
  I2C_ucg_font_fur25,
  I2C_ucg_font_fur25_hf,
  I2C_ucg_font_fur25_hn,
  I2C_ucg_font_fur25_hr,
  I2C_ucg_font_fur25n,
  I2C_ucg_font_fur25r,
  I2C_ucg_font_fur25_tf,
  I2C_ucg_font_fur25_tn,
  I2C_ucg_font_fur25_tr,
  I2C_ucg_font_fur30,
  I2C_ucg_font_fur30_hf,
  I2C_ucg_font_fur30_hn,
  I2C_ucg_font_fur30_hr,
  I2C_ucg_font_fur30n,
  I2C_ucg_font_fur30r,
  I2C_ucg_font_fur30_tf,
  I2C_ucg_font_fur30_tn,
  I2C_ucg_font_fur30_tr,
  I2C_ucg_font_fur35_hf,
  I2C_ucg_font_fur35_hn,
  I2C_ucg_font_fur35_hr,
  I2C_ucg_font_fur35n,
  I2C_ucg_font_fur35_tf,
  I2C_ucg_font_fur35_tn,
  I2C_ucg_font_fur35_tr,
  I2C_ucg_font_fur42_hf,
  I2C_ucg_font_fur42_hn,
  I2C_ucg_font_fur42_hr,
  I2C_ucg_font_fur42n,
  I2C_ucg_font_fur42_tf,
  I2C_ucg_font_fur42_tn,
  I2C_ucg_font_fur42_tr,
  I2C_ucg_font_fur49_hn,
  I2C_ucg_font_fur49n,
  I2C_ucg_font_fur49_tn,
  I2C_ucg_font_fur49_tr,
  I2C_ucg_font_helvB08,
  I2C_ucg_font_helvB08_hf,
  I2C_ucg_font_helvB08_hr,
  I2C_ucg_font_helvB08r,
  I2C_ucg_font_helvB08_tf,
  I2C_ucg_font_helvB08_tr,
  I2C_ucg_font_helvB10,
  I2C_ucg_font_helvB10_hf,
  I2C_ucg_font_helvB10_hr,
  I2C_ucg_font_helvB10r,
  I2C_ucg_font_helvB10_tf,
  I2C_ucg_font_helvB10_tr,
  I2C_ucg_font_helvB12,
  I2C_ucg_font_helvB12_hf,
  I2C_ucg_font_helvB12_hr,
  I2C_ucg_font_helvB12r,
  I2C_ucg_font_helvB12_tf,
  I2C_ucg_font_helvB12_tr,
  I2C_ucg_font_helvB14,
  I2C_ucg_font_helvB14_hf,
  I2C_ucg_font_helvB14_hr,
  I2C_ucg_font_helvB14r,
  I2C_ucg_font_helvB14_tf,
  I2C_ucg_font_helvB14_tr,
  I2C_ucg_font_helvB18,
  I2C_ucg_font_helvB18_hf,
  I2C_ucg_font_helvB18_hr,
  I2C_ucg_font_helvB18r,
  I2C_ucg_font_helvB18_tf,
  I2C_ucg_font_helvB18_tr,
  I2C_ucg_font_helvB24,
  I2C_ucg_font_helvB24_hf,
  I2C_ucg_font_helvB24_hn,
  I2C_ucg_font_helvB24_hr,
  I2C_ucg_font_helvB24n,
  I2C_ucg_font_helvB24r,
  I2C_ucg_font_helvB24_tf,
  I2C_ucg_font_helvB24_tn,
  I2C_ucg_font_helvB24_tr,
  I2C_ucg_font_helvR08,
  I2C_ucg_font_helvR08_hf,
  I2C_ucg_font_helvR08_hr,
  I2C_ucg_font_helvR08r,
  I2C_ucg_font_helvR08_tf,
  I2C_ucg_font_helvR08_tr,
  I2C_ucg_font_helvR10,
  I2C_ucg_font_helvR10_hf,
  I2C_ucg_font_helvR10_hr,
  I2C_ucg_font_helvR10r,
  I2C_ucg_font_helvR10_tf,
  I2C_ucg_font_helvR10_tr,
  I2C_ucg_font_helvR12,
  I2C_ucg_font_helvR12_hf,
  I2C_ucg_font_helvR12_hr,
  I2C_ucg_font_helvR12r,
  I2C_ucg_font_helvR12_tf,
  I2C_ucg_font_helvR12_tr,
  I2C_ucg_font_helvR14,
  I2C_ucg_font_helvR14_hf,
  I2C_ucg_font_helvR14_hr,
  I2C_ucg_font_helvR14r,
  I2C_ucg_font_helvR14_tf,
  I2C_ucg_font_helvR14_tr,
  I2C_ucg_font_helvR18,
  I2C_ucg_font_helvR18_hf,
  I2C_ucg_font_helvR18_hr,
  I2C_ucg_font_helvR18r,
  I2C_ucg_font_helvR18_tf,
  I2C_ucg_font_helvR18_tr,
  I2C_ucg_font_helvR24,
  I2C_ucg_font_helvR24_hf,
  I2C_ucg_font_helvR24_hn,
  I2C_ucg_font_helvR24_hr,
  I2C_ucg_font_helvR24n,
  I2C_ucg_font_helvR24r,
  I2C_ucg_font_helvR24_tf,
  I2C_ucg_font_helvR24_tn,
  I2C_ucg_font_helvR24_tr,
  I2C_ucg_font_inb16_mf,
  I2C_ucg_font_inb16_mn,
  I2C_ucg_font_inb16_mr,
  I2C_ucg_font_inb16_tf,
  I2C_ucg_font_inb16_tn,
  I2C_ucg_font_inb16_tr,
  I2C_ucg_font_inb19_mf,
  I2C_ucg_font_inb19_mn,
  I2C_ucg_font_inb19_mr,
  I2C_ucg_font_inb19_tf,
  I2C_ucg_font_inb19_tn,
  I2C_ucg_font_inb19_tr,
  I2C_ucg_font_inb21_mf,
  I2C_ucg_font_inb21_mn,
  I2C_ucg_font_inb21_mr,
  I2C_ucg_font_inb21_tf,
  I2C_ucg_font_inb21_tn,
  I2C_ucg_font_inb21_tr,
  I2C_ucg_font_inb24_mf,
  I2C_ucg_font_inb24_mn,
  I2C_ucg_font_inb24_mr,
  I2C_ucg_font_inb24_tf,
  I2C_ucg_font_inb24_tn,
  I2C_ucg_font_inb24_tr,
  I2C_ucg_font_inb27_mf,
  I2C_ucg_font_inb27_mn,
  I2C_ucg_font_inb27_mr,
  I2C_ucg_font_inb27_tf,
  I2C_ucg_font_inb27_tn,
  I2C_ucg_font_inb27_tr,
  I2C_ucg_font_inb30_mf,
  I2C_ucg_font_inb30_mn,
  I2C_ucg_font_inb30_mr,
  I2C_ucg_font_inb30_tf,
  I2C_ucg_font_inb30_tn,
  I2C_ucg_font_inb30_tr,
  I2C_ucg_font_inb33_mf,
  I2C_ucg_font_inb33_mn,
  I2C_ucg_font_inb33_mr,
  I2C_ucg_font_inb33_tf,
  I2C_ucg_font_inb33_tn,
  I2C_ucg_font_inb33_tr,
  I2C_ucg_font_inb38_mf,
  I2C_ucg_font_inb38_mn,
  I2C_ucg_font_inb38_mr,
  I2C_ucg_font_inb38_tf,
  I2C_ucg_font_inb38_tn,
  I2C_ucg_font_inb38_tr,
  I2C_ucg_font_inb42_mf,
  I2C_ucg_font_inb42_mn,
  I2C_ucg_font_inb42_mr,
  I2C_ucg_font_inb42_tf,
  I2C_ucg_font_inb42_tn,
  I2C_ucg_font_inb42_tr,
  I2C_ucg_font_inb46_mf,
  I2C_ucg_font_inb46_mn,
  I2C_ucg_font_inb46_mr,
  I2C_ucg_font_inb46_tf,
  I2C_ucg_font_inb46_tn,
  I2C_ucg_font_inb46_tr,
  I2C_ucg_font_inb49_mf,
  I2C_ucg_font_inb49_mn,
  I2C_ucg_font_inb49_mr,
  I2C_ucg_font_inb49_tf,
  I2C_ucg_font_inb49_tn,
  I2C_ucg_font_inb49_tr,
  I2C_ucg_font_inb53_mf,
  I2C_ucg_font_inb53_mn,
  I2C_ucg_font_inb53_mr,
  I2C_ucg_font_inb53_tf,
  I2C_ucg_font_inb53_tn,
  I2C_ucg_font_inb53_tr,
  I2C_ucg_font_inb57_mf,
  I2C_ucg_font_inb57_mn,
  I2C_ucg_font_inb57_mr,
  I2C_ucg_font_inb57_tf,
  I2C_ucg_font_inb57_tn,
  I2C_ucg_font_inb57_tr,
  I2C_ucg_font_inb63_mn,
  I2C_ucg_font_inb63_tf,
  I2C_ucg_font_inb63_tn,
  I2C_ucg_font_inb63_tr,
  I2C_ucg_font_inr16_mf,
  I2C_ucg_font_inr16_mn,
  I2C_ucg_font_inr16_mr,
  I2C_ucg_font_inr16_tf,
  I2C_ucg_font_inr16_tn,
  I2C_ucg_font_inr16_tr,
  I2C_ucg_font_inr19_mf,
  I2C_ucg_font_inr19_mn,
  I2C_ucg_font_inr19_mr,
  I2C_ucg_font_inr19_tf,
  I2C_ucg_font_inr19_tn,
  I2C_ucg_font_inr19_tr,
  I2C_ucg_font_inr21_mf,
  I2C_ucg_font_inr21_mn,
  I2C_ucg_font_inr21_mr,
  I2C_ucg_font_inr21_tf,
  I2C_ucg_font_inr21_tn,
  I2C_ucg_font_inr21_tr,
  I2C_ucg_font_inr24_mf,
  I2C_ucg_font_inr24_mn,
  I2C_ucg_font_inr24_mr,
  I2C_ucg_font_inr24_tf,
  I2C_ucg_font_inr24_tn,
  I2C_ucg_font_inr24_tr,
  I2C_ucg_font_inr27_mf,
  I2C_ucg_font_inr27_mn,
  I2C_ucg_font_inr27_mr,
  I2C_ucg_font_inr27_tf,
  I2C_ucg_font_inr27_tn,
  I2C_ucg_font_inr27_tr,
  I2C_ucg_font_inr30_mf,
  I2C_ucg_font_inr30_mn,
  I2C_ucg_font_inr30_mr,
  I2C_ucg_font_inr30_tf,
  I2C_ucg_font_inr30_tn,
  I2C_ucg_font_inr30_tr,
  I2C_ucg_font_inr33_mf,
  I2C_ucg_font_inr33_mn,
  I2C_ucg_font_inr33_mr,
  I2C_ucg_font_inr33_tf,
  I2C_ucg_font_inr33_tn,
  I2C_ucg_font_inr33_tr,
  I2C_ucg_font_inr38_mf,
  I2C_ucg_font_inr38_mn,
  I2C_ucg_font_inr38_mr,
  I2C_ucg_font_inr38_tf,
  I2C_ucg_font_inr38_tn,
  I2C_ucg_font_inr38_tr,
  I2C_ucg_font_inr42_mf,
  I2C_ucg_font_inr42_mn,
  I2C_ucg_font_inr42_mr,
  I2C_ucg_font_inr42_tf,
  I2C_ucg_font_inr42_tn,
  I2C_ucg_font_inr42_tr,
  I2C_ucg_font_inr46_mf,
  I2C_ucg_font_inr46_mn,
  I2C_ucg_font_inr46_mr,
  I2C_ucg_font_inr46_tf,
  I2C_ucg_font_inr46_tn,
  I2C_ucg_font_inr46_tr,
  I2C_ucg_font_inr49_mf,
  I2C_ucg_font_inr49_mn,
  I2C_ucg_font_inr49_mr,
  I2C_ucg_font_inr49_tf,
  I2C_ucg_font_inr49_tn,
  I2C_ucg_font_inr49_tr,
  I2C_ucg_font_inr53_mf,
  I2C_ucg_font_inr53_mn,
  I2C_ucg_font_inr53_mr,
  I2C_ucg_font_inr53_tf,
  I2C_ucg_font_inr53_tn,
  I2C_ucg_font_inr53_tr,
  I2C_ucg_font_inr57_mn,
  I2C_ucg_font_inr57_tf,
  I2C_ucg_font_inr57_tn,
  I2C_ucg_font_inr57_tr,
  I2C_ucg_font_inr62_mn,
  I2C_ucg_font_inr62_tn,
  I2C_ucg_font_inr62_tr,
  I2C_ucg_font_logisoso16,
  I2C_ucg_font_logisoso16_hf,
  I2C_ucg_font_logisoso16_hn,
  I2C_ucg_font_logisoso16_hr,
  I2C_ucg_font_logisoso16n,
  I2C_ucg_font_logisoso16r,
  I2C_ucg_font_logisoso16_tf,
  I2C_ucg_font_logisoso16_tn,
  I2C_ucg_font_logisoso16_tr,
  I2C_ucg_font_logisoso18,
  I2C_ucg_font_logisoso18_hf,
  I2C_ucg_font_logisoso18_hn,
  I2C_ucg_font_logisoso18_hr,
  I2C_ucg_font_logisoso18n,
  I2C_ucg_font_logisoso18r,
  I2C_ucg_font_logisoso18_tf,
  I2C_ucg_font_logisoso18_tn,
  I2C_ucg_font_logisoso18_tr,
  I2C_ucg_font_logisoso20,
  I2C_ucg_font_logisoso20_hf,
  I2C_ucg_font_logisoso20_hn,
  I2C_ucg_font_logisoso20_hr,
  I2C_ucg_font_logisoso20n,
  I2C_ucg_font_logisoso20r,
  I2C_ucg_font_logisoso20_tf,
  I2C_ucg_font_logisoso20_tn,
  I2C_ucg_font_logisoso20_tr,
  I2C_ucg_font_logisoso22,
  I2C_ucg_font_logisoso22_hf,
  I2C_ucg_font_logisoso22_hn,
  I2C_ucg_font_logisoso22_hr,
  I2C_ucg_font_logisoso22n,
  I2C_ucg_font_logisoso22r,
  I2C_ucg_font_logisoso22_tf,
  I2C_ucg_font_logisoso22_tn,
  I2C_ucg_font_logisoso22_tr,
  I2C_ucg_font_logisoso24,
  I2C_ucg_font_logisoso24_hf,
  I2C_ucg_font_logisoso24_hn,
  I2C_ucg_font_logisoso24_hr,
  I2C_ucg_font_logisoso24n,
  I2C_ucg_font_logisoso24r,
  I2C_ucg_font_logisoso24_tf,
  I2C_ucg_font_logisoso24_tn,
  I2C_ucg_font_logisoso24_tr,
  I2C_ucg_font_logisoso26,
  I2C_ucg_font_logisoso26_hf,
  I2C_ucg_font_logisoso26_hn,
  I2C_ucg_font_logisoso26_hr,
  I2C_ucg_font_logisoso26n,
  I2C_ucg_font_logisoso26r,
  I2C_ucg_font_logisoso26_tf,
  I2C_ucg_font_logisoso26_tn,
  I2C_ucg_font_logisoso26_tr,
  I2C_ucg_font_logisoso28,
  I2C_ucg_font_logisoso28_hf,
  I2C_ucg_font_logisoso28_hn,
  I2C_ucg_font_logisoso28_hr,
  I2C_ucg_font_logisoso28n,
  I2C_ucg_font_logisoso28r,
  I2C_ucg_font_logisoso28_tf,
  I2C_ucg_font_logisoso28_tn,
  I2C_ucg_font_logisoso28_tr,
  I2C_ucg_font_logisoso30,
  I2C_ucg_font_logisoso30n,
  I2C_ucg_font_logisoso30r,
  I2C_ucg_font_logisoso30_tf,
  I2C_ucg_font_logisoso30_tn,
  I2C_ucg_font_logisoso30_tr,
  I2C_ucg_font_logisoso32,
  I2C_ucg_font_logisoso32n,
  I2C_ucg_font_logisoso32r,
  I2C_ucg_font_logisoso32_tf,
  I2C_ucg_font_logisoso32_tn,
  I2C_ucg_font_logisoso32_tr,
  I2C_ucg_font_logisoso34,
  I2C_ucg_font_logisoso34n,
  I2C_ucg_font_logisoso34r,
  I2C_ucg_font_logisoso34_tf,
  I2C_ucg_font_logisoso34_tn,
  I2C_ucg_font_logisoso34_tr,
  I2C_ucg_font_logisoso38,
  I2C_ucg_font_logisoso38n,
  I2C_ucg_font_logisoso38r,
  I2C_ucg_font_logisoso38_tf,
  I2C_ucg_font_logisoso38_tn,
  I2C_ucg_font_logisoso38_tr,
  I2C_ucg_font_logisoso42,
  I2C_ucg_font_logisoso42n,
  I2C_ucg_font_logisoso42r,
  I2C_ucg_font_logisoso42_tf,
  I2C_ucg_font_logisoso42_tn,
  I2C_ucg_font_logisoso42_tr,
  I2C_ucg_font_logisoso46n,
  I2C_ucg_font_logisoso46r,
  I2C_ucg_font_logisoso46_tf,
  I2C_ucg_font_logisoso46_tn,
  I2C_ucg_font_logisoso46_tr,
  I2C_ucg_font_logisoso50n,
  I2C_ucg_font_logisoso50r,
  I2C_ucg_font_logisoso50_tf,
  I2C_ucg_font_logisoso50_tn,
  I2C_ucg_font_logisoso50_tr,
  I2C_ucg_font_logisoso54n,
  I2C_ucg_font_logisoso54_tf,
  I2C_ucg_font_logisoso54_tn,
  I2C_ucg_font_logisoso54_tr,
  I2C_ucg_font_logisoso58n,
  I2C_ucg_font_logisoso58_tf,
  I2C_ucg_font_logisoso58_tn,
  I2C_ucg_font_logisoso58_tr,
  I2C_ucg_font_logisoso62n,
  I2C_ucg_font_logisoso62_tn,
  I2C_ucg_font_lucasfont_alternate,
  I2C_ucg_font_lucasfont_alternate_hf,
  I2C_ucg_font_lucasfont_alternate_hn,
  I2C_ucg_font_lucasfont_alternate_hr,
  I2C_ucg_font_lucasfont_alternaten,
  I2C_ucg_font_lucasfont_alternater,
  I2C_ucg_font_lucasfont_alternate_tf,
  I2C_ucg_font_lucasfont_alternate_tn,
  I2C_ucg_font_lucasfont_alternate_tr,
  I2C_ucg_font_micro,
  I2C_ucg_font_micro_mf,
  I2C_ucg_font_micro_tf,
  I2C_ucg_font_ncenB08,
  I2C_ucg_font_ncenB08_hf,
  I2C_ucg_font_ncenB08_hr,
  I2C_ucg_font_ncenB08r,
  I2C_ucg_font_ncenB08_tf,
  I2C_ucg_font_ncenB08_tr,
  I2C_ucg_font_ncenB10,
  I2C_ucg_font_ncenB10_hf,
  I2C_ucg_font_ncenB10_hr,
  I2C_ucg_font_ncenB10r,
  I2C_ucg_font_ncenB10_tf,
  I2C_ucg_font_ncenB10_tr,
  I2C_ucg_font_ncenB12,
  I2C_ucg_font_ncenB12_hf,
  I2C_ucg_font_ncenB12_hr,
  I2C_ucg_font_ncenB12r,
  I2C_ucg_font_ncenB12_tf,
  I2C_ucg_font_ncenB12_tr,
  I2C_ucg_font_ncenB14,
  I2C_ucg_font_ncenB14_hf,
  I2C_ucg_font_ncenB14_hr,
  I2C_ucg_font_ncenB14r,
  I2C_ucg_font_ncenB14_tf,
  I2C_ucg_font_ncenB14_tr,
  I2C_ucg_font_ncenB18,
  I2C_ucg_font_ncenB18_hf,
  I2C_ucg_font_ncenB18_hr,
  I2C_ucg_font_ncenB18r,
  I2C_ucg_font_ncenB18_tf,
  I2C_ucg_font_ncenB18_tr,
  I2C_ucg_font_ncenB24,
  I2C_ucg_font_ncenB24_hf,
  I2C_ucg_font_ncenB24_hn,
  I2C_ucg_font_ncenB24_hr,
  I2C_ucg_font_ncenB24n,
  I2C_ucg_font_ncenB24r,
  I2C_ucg_font_ncenB24_tf,
  I2C_ucg_font_ncenB24_tn,
  I2C_ucg_font_ncenB24_tr,
  I2C_ucg_font_ncenR08,
  I2C_ucg_font_ncenR08_hf,
  I2C_ucg_font_ncenR08_hr,
  I2C_ucg_font_ncenR08r,
  I2C_ucg_font_ncenR08_tf,
  I2C_ucg_font_ncenR08_tr,
  I2C_ucg_font_ncenR10,
  I2C_ucg_font_ncenR10_hf,
  I2C_ucg_font_ncenR10_hr,
  I2C_ucg_font_ncenR10r,
  I2C_ucg_font_ncenR10_tf,
  I2C_ucg_font_ncenR10_tr,
  I2C_ucg_font_ncenR12,
  I2C_ucg_font_ncenR12_hf,
  I2C_ucg_font_ncenR12_hr,
  I2C_ucg_font_ncenR12r,
  I2C_ucg_font_ncenR12_tf,
  I2C_ucg_font_ncenR12_tr,
  I2C_ucg_font_ncenR14,
  I2C_ucg_font_ncenR14_hf,
  I2C_ucg_font_ncenR14_hr,
  I2C_ucg_font_ncenR14r,
  I2C_ucg_font_ncenR14_tf,
  I2C_ucg_font_ncenR14_tr,
  I2C_ucg_font_ncenR18,
  I2C_ucg_font_ncenR18_hf,
  I2C_ucg_font_ncenR18_hr,
  I2C_ucg_font_ncenR18r,
  I2C_ucg_font_ncenR18_tf,
  I2C_ucg_font_ncenR18_tr,
  I2C_ucg_font_ncenR24,
  I2C_ucg_font_ncenR24_hf,
  I2C_ucg_font_ncenR24_hn,
  I2C_ucg_font_ncenR24_hr,
  I2C_ucg_font_ncenR24n,
  I2C_ucg_font_ncenR24r,
  I2C_ucg_font_ncenR24_tf,
  I2C_ucg_font_ncenR24_tn,
  I2C_ucg_font_ncenR24_tr,
  I2C_ucg_font_orgv01,
  I2C_ucg_font_orgv01_hf,
  I2C_ucg_font_orgv01_hn,
  I2C_ucg_font_orgv01_hr,
  I2C_ucg_font_orgv01n,
  I2C_ucg_font_orgv01r,
  I2C_ucg_font_orgv01_tf,
  I2C_ucg_font_orgv01_tn,
  I2C_ucg_font_orgv01_tr,
  I2C_ucg_font_osb18,
  I2C_ucg_font_osb18_hf,
  I2C_ucg_font_osb18_hn,
  I2C_ucg_font_osb18_hr,
  I2C_ucg_font_osb18n,
  I2C_ucg_font_osb18r,
  I2C_ucg_font_osb18_tf,
  I2C_ucg_font_osb18_tn,
  I2C_ucg_font_osb18_tr,
  I2C_ucg_font_osb21,
  I2C_ucg_font_osb21_hf,
  I2C_ucg_font_osb21_hn,
  I2C_ucg_font_osb21_hr,
  I2C_ucg_font_osb21n,
  I2C_ucg_font_osb21r,
  I2C_ucg_font_osb21_tf,
  I2C_ucg_font_osb21_tn,
  I2C_ucg_font_osb21_tr,
  I2C_ucg_font_osb26,
  I2C_ucg_font_osb26_hf,
  I2C_ucg_font_osb26_hn,
  I2C_ucg_font_osb26_hr,
  I2C_ucg_font_osb26n,
  I2C_ucg_font_osb26r,
  I2C_ucg_font_osb26_tf,
  I2C_ucg_font_osb26_tn,
  I2C_ucg_font_osb26_tr,
  I2C_ucg_font_osb29,
  I2C_ucg_font_osb29_hf,
  I2C_ucg_font_osb29_hn,
  I2C_ucg_font_osb29_hr,
  I2C_ucg_font_osb29n,
  I2C_ucg_font_osb29r,
  I2C_ucg_font_osb29_tf,
  I2C_ucg_font_osb29_tn,
  I2C_ucg_font_osb29_tr,
  I2C_ucg_font_osb35,
  I2C_ucg_font_osb35_hf,
  I2C_ucg_font_osb35_hn,
  I2C_ucg_font_osb35_hr,
  I2C_ucg_font_osb35n,
  I2C_ucg_font_osb35r,
  I2C_ucg_font_osb35_tf,
  I2C_ucg_font_osb35_tn,
  I2C_ucg_font_osb35_tr,
  I2C_ucg_font_osb41_hf,
  I2C_ucg_font_osb41_hn,
  I2C_ucg_font_osb41_hr,
  I2C_ucg_font_osb41_tf,
  I2C_ucg_font_osb41_tn,
  I2C_ucg_font_osb41_tr,
  I2C_ucg_font_osr18,
  I2C_ucg_font_osr18_hf,
  I2C_ucg_font_osr18_hn,
  I2C_ucg_font_osr18_hr,
  I2C_ucg_font_osr18n,
  I2C_ucg_font_osr18r,
  I2C_ucg_font_osr18_tf,
  I2C_ucg_font_osr18_tn,
  I2C_ucg_font_osr18_tr,
  I2C_ucg_font_osr21,
  I2C_ucg_font_osr21_hf,
  I2C_ucg_font_osr21_hn,
  I2C_ucg_font_osr21_hr,
  I2C_ucg_font_osr21n,
  I2C_ucg_font_osr21r,
  I2C_ucg_font_osr21_tf,
  I2C_ucg_font_osr21_tn,
  I2C_ucg_font_osr21_tr,
  I2C_ucg_font_osr26,
  I2C_ucg_font_osr26_hf,
  I2C_ucg_font_osr26_hn,
  I2C_ucg_font_osr26_hr,
  I2C_ucg_font_osr26n,
  I2C_ucg_font_osr26r,
  I2C_ucg_font_osr26_tf,
  I2C_ucg_font_osr26_tn,
  I2C_ucg_font_osr26_tr,
  I2C_ucg_font_osr29,
  I2C_ucg_font_osr29_hf,
  I2C_ucg_font_osr29_hn,
  I2C_ucg_font_osr29_hr,
  I2C_ucg_font_osr29n,
  I2C_ucg_font_osr29r,
  I2C_ucg_font_osr29_tf,
  I2C_ucg_font_osr29_tn,
  I2C_ucg_font_osr29_tr,
  I2C_ucg_font_osr35,
  I2C_ucg_font_osr35_hf,
  I2C_ucg_font_osr35_hn,
  I2C_ucg_font_osr35_hr,
  I2C_ucg_font_osr35n,
  I2C_ucg_font_osr35r,
  I2C_ucg_font_osr35_tf,
  I2C_ucg_font_osr35_tn,
  I2C_ucg_font_osr35_tr,
  I2C_ucg_font_osr41_hf,
  I2C_ucg_font_osr41_hn,
  I2C_ucg_font_osr41_hr,
  I2C_ucg_font_osr41_tf,
  I2C_ucg_font_osr41_tn,
  I2C_ucg_font_osr41_tr,
  I2C_ucg_font_p01type,
  I2C_ucg_font_p01type_hf,
  I2C_ucg_font_p01type_hn,
  I2C_ucg_font_p01type_hr,
  I2C_ucg_font_p01typen,
  I2C_ucg_font_p01typer,
  I2C_ucg_font_p01type_tf,
  I2C_ucg_font_p01type_tn,
  I2C_ucg_font_p01type_tr,
  I2C_ucg_font_pixelle_micro,
  I2C_ucg_font_pixelle_micro_hf,
  I2C_ucg_font_pixelle_micro_hn,
  I2C_ucg_font_pixelle_micro_hr,
  I2C_ucg_font_pixelle_micron,
  I2C_ucg_font_pixelle_micror,
  I2C_ucg_font_pixelle_micro_tf,
  I2C_ucg_font_pixelle_micro_tn,
  I2C_ucg_font_pixelle_micro_tr,
  I2C_ucg_font_profont10,
  I2C_ucg_font_profont10_8f,
  I2C_ucg_font_profont10_8r,
  I2C_ucg_font_profont10_mf,
  I2C_ucg_font_profont10_mr,
  I2C_ucg_font_profont10r,
  I2C_ucg_font_profont11,
  I2C_ucg_font_profont11_8f,
  I2C_ucg_font_profont11_8r,
  I2C_ucg_font_profont11_mf,
  I2C_ucg_font_profont11_mr,
  I2C_ucg_font_profont11r,
  I2C_ucg_font_profont12,
  I2C_ucg_font_profont12_8f,
  I2C_ucg_font_profont12_8r,
  I2C_ucg_font_profont12_mf,
  I2C_ucg_font_profont12_mr,
  I2C_ucg_font_profont12r,
  I2C_ucg_font_profont15,
  I2C_ucg_font_profont15_8f,
  I2C_ucg_font_profont15_8r,
  I2C_ucg_font_profont15_mf,
  I2C_ucg_font_profont15_mr,
  I2C_ucg_font_profont15r,
  I2C_ucg_font_profont17,
  I2C_ucg_font_profont17_8f,
  I2C_ucg_font_profont17_8r,
  I2C_ucg_font_profont17_mf,
  I2C_ucg_font_profont17_mr,
  I2C_ucg_font_profont17r,
  I2C_ucg_font_profont22,
  I2C_ucg_font_profont22_8f,
  I2C_ucg_font_profont22_8n,
  I2C_ucg_font_profont22_8r,
  I2C_ucg_font_profont22_mf,
  I2C_ucg_font_profont22_mn,
  I2C_ucg_font_profont22_mr,
  I2C_ucg_font_profont22n,
  I2C_ucg_font_profont22r,
  I2C_ucg_font_profont29,
  I2C_ucg_font_profont29_8f,
  I2C_ucg_font_profont29_8n,
  I2C_ucg_font_profont29_8r,
  I2C_ucg_font_profont29_mf,
  I2C_ucg_font_profont29_mn,
  I2C_ucg_font_profont29_mr,
  I2C_ucg_font_profont29n,
  I2C_ucg_font_profont29r,
  I2C_ucg_font_robot_de_niro,
  I2C_ucg_font_robot_de_niro_hf,
  I2C_ucg_font_robot_de_niro_hn,
  I2C_ucg_font_robot_de_niro_hr,
  I2C_ucg_font_robot_de_niron,
  I2C_ucg_font_robot_de_niror,
  I2C_ucg_font_robot_de_niro_tf,
  I2C_ucg_font_robot_de_niro_tn,
  I2C_ucg_font_robot_de_niro_tr,
  I2C_ucg_font_symb08,
  I2C_ucg_font_symb08r,
  I2C_ucg_font_symb08_tf,
  I2C_ucg_font_symb08_tr,
  I2C_ucg_font_symb10,
  I2C_ucg_font_symb10r,
  I2C_ucg_font_symb10_tf,
  I2C_ucg_font_symb10_tr,
  I2C_ucg_font_symb12,
  I2C_ucg_font_symb12r,
  I2C_ucg_font_symb12_tf,
  I2C_ucg_font_symb12_tr,
  I2C_ucg_font_symb14,
  I2C_ucg_font_symb14r,
  I2C_ucg_font_symb14_tf,
  I2C_ucg_font_symb14_tr,
  I2C_ucg_font_symb18,
  I2C_ucg_font_symb18r,
  I2C_ucg_font_symb18_tf,
  I2C_ucg_font_symb18_tr,
  I2C_ucg_font_symb24,
  I2C_ucg_font_symb24r,
  I2C_ucg_font_symb24_tf,
  I2C_ucg_font_symb24_tr,
  I2C_ucg_font_timB08,
  I2C_ucg_font_timB08_hf,
  I2C_ucg_font_timB08_hr,
  I2C_ucg_font_timB08r,
  I2C_ucg_font_timB08_tf,
  I2C_ucg_font_timB08_tr,
  I2C_ucg_font_timB10,
  I2C_ucg_font_timB10_hf,
  I2C_ucg_font_timB10_hr,
  I2C_ucg_font_timB10r,
  I2C_ucg_font_timB10_tf,
  I2C_ucg_font_timB10_tr,
  I2C_ucg_font_timB12,
  I2C_ucg_font_timB12_hf,
  I2C_ucg_font_timB12_hr,
  I2C_ucg_font_timB12r,
  I2C_ucg_font_timB12_tf,
  I2C_ucg_font_timB12_tr,
  I2C_ucg_font_timB14,
  I2C_ucg_font_timB14_hf,
  I2C_ucg_font_timB14_hr,
  I2C_ucg_font_timB14r,
  I2C_ucg_font_timB14_tf,
  I2C_ucg_font_timB14_tr,
  I2C_ucg_font_timB18,
  I2C_ucg_font_timB18_hf,
  I2C_ucg_font_timB18_hr,
  I2C_ucg_font_timB18r,
  I2C_ucg_font_timB18_tf,
  I2C_ucg_font_timB18_tr,
  I2C_ucg_font_timB24,
  I2C_ucg_font_timB24_hf,
  I2C_ucg_font_timB24_hn,
  I2C_ucg_font_timB24_hr,
  I2C_ucg_font_timB24n,
  I2C_ucg_font_timB24r,
  I2C_ucg_font_timB24_tf,
  I2C_ucg_font_timB24_tn,
  I2C_ucg_font_timB24_tr,
  I2C_ucg_font_timR08,
  I2C_ucg_font_timR08_hf,
  I2C_ucg_font_timR08_hr,
  I2C_ucg_font_timR08r,
  I2C_ucg_font_timR08_tf,
  I2C_ucg_font_timR08_tr,
  I2C_ucg_font_timR10,
  I2C_ucg_font_timR10_hf,
  I2C_ucg_font_timR10_hr,
  I2C_ucg_font_timR10r,
  I2C_ucg_font_timR10_tf,
  I2C_ucg_font_timR10_tr,
  I2C_ucg_font_timR12,
  I2C_ucg_font_timR12_hf,
  I2C_ucg_font_timR12_hr,
  I2C_ucg_font_timR12r,
  I2C_ucg_font_timR12_tf,
  I2C_ucg_font_timR12_tr,
  I2C_ucg_font_timR14,
  I2C_ucg_font_timR14_hf,
  I2C_ucg_font_timR14_hr,
  I2C_ucg_font_timR14r,
  I2C_ucg_font_timR14_tf,
  I2C_ucg_font_timR14_tr,
  I2C_ucg_font_timR18,
  I2C_ucg_font_timR18_hf,
  I2C_ucg_font_timR18_hr,
  I2C_ucg_font_timR18r,
  I2C_ucg_font_timR18_tf,
  I2C_ucg_font_timR18_tr,
  I2C_ucg_font_timR24,
  I2C_ucg_font_timR24_hf,
  I2C_ucg_font_timR24_hn,
  I2C_ucg_font_timR24_hr,
  I2C_ucg_font_timR24n,
  I2C_ucg_font_timR24r,
  I2C_ucg_font_timR24_tf,
  I2C_ucg_font_timR24_tn,
  I2C_ucg_font_timR24_tr,
  I2C_ucg_font_tpss,
  I2C_ucg_font_tpssb,
  I2C_ucg_font_tpssb_hf,
  I2C_ucg_font_tpssb_hn,
  I2C_ucg_font_tpssb_hr,
  I2C_ucg_font_tpssbn,
  I2C_ucg_font_tpssbr,
  I2C_ucg_font_tpssb_tf,
  I2C_ucg_font_tpssb_tn,
  I2C_ucg_font_tpssb_tr,
  I2C_ucg_font_tpss_hf,
  I2C_ucg_font_tpss_hn,
  I2C_ucg_font_tpss_hr,
  I2C_ucg_font_tpssn,
  I2C_ucg_font_tpssr,
  I2C_ucg_font_tpss_tf,
  I2C_ucg_font_tpss_tn,
  I2C_ucg_font_tpss_tr,
  I2C_ucg_font_trixel_square,
  I2C_ucg_font_trixel_square_hf,
  I2C_ucg_font_trixel_square_hn,
  I2C_ucg_font_trixel_square_hr,
  I2C_ucg_font_trixel_squaren,
  I2C_ucg_font_trixel_squarer,
  I2C_ucg_font_trixel_square_tf,
  I2C_ucg_font_trixel_square_tn,
  I2C_ucg_font_trixel_square_tr,
  I2C_ucg_font_u8glib_4,
  I2C_ucg_font_u8glib_4_hf,
  I2C_ucg_font_u8glib_4_hr,
  I2C_ucg_font_u8glib_4r,
  I2C_ucg_font_u8glib_4_tf,
  I2C_ucg_font_u8glib_4_tr,
  I2C_ucg_font_unifont,
  I2C_ucg_font_unifont_75r,
  I2C_ucg_font_unifont_76,
  I2C_ucg_font_unifont_77,
  I2C_ucg_font_unifont_86,
  I2C_ucg_font_unifont_mf,
  I2C_ucg_font_unifont_mr,
  I2C_ucg_font_unifontr,
  I2C_ucg_font_unifont_tf,
  I2C_ucg_font_unifont_tr
};

#endif

